
-- AVOID GAME EXAMPLE --
-- by MadByte --

local screenWidth, screenHeight = love.graphics.getDimensions()
love.window.setMode(screenWidth, screenHeight, {vsync=false})




--------------------------------------------------------------------------------------------
-- player Box --
--------------------------------------------------------------------------------------------
local player = {}

function player:init()
  self.width    = 32
  self.height   = 32
  self.x        = screenWidth/2-self.width/2
  self.y        = screenHeight - 100
  self.velocity = 300
  self.removed  = false
end


function player:moveByKey(dt)
  if love.keyboard.isDown("left") and self.x > 0 then
    self.x = self.x - self.velocity * dt
  elseif love.keyboard.isDown("right") and self.x + self.width < screenWidth then
    self.x = self.x + self.velocity * dt
  end
  
  if love.keyboard.isDown("up") and self.y > 0 then
    self.y = self.y - self.velocity * dt
  elseif love.keyboard.isDown("down") and self.y + self.height < screenHeight then
    self.y = self.y + self.velocity * dt
  end
end

  
function player:update(dt)
  -- Key Movement --
  self:moveByKey(dt)
end


function player:draw()
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
end




--------------------------------------------------------------------------------------------
-- Other Boxes --
--------------------------------------------------------------------------------------------
local boxes = {}

function boxes:init()
  self.objects = {}
  self.timer = 0
  self.spawnRate = .3
end


function boxes:addBox()
  local box = {}
  box.width       = 32
  box.height      = 32
  box.x           = love.math.random(0, screenWidth-box.width)
  box.y           = -box.height
  box.xVelocity   = love.math.random(-50, 50)
  box.yVelocity   = love.math.random(100, 500)
  box.removed     = false
  
  function box:update(dt)
    self.x = self.x + self.xVelocity * dt
    self.y = self.y + self.yVelocity * dt
    if self.y > screenHeight then self.removed = true end
  end
  
  
  function box:draw()
    love.graphics.setColor(255, 50, 100)
    love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
    love.graphics.setColor(255, 255, 255)
  end
  
  table.insert(boxes.objects, box)
end


function boxes:update(dt)
  -- Spawn Logic --
  self.timer = self.timer + dt
  if self.timer > self.spawnRate then
    self:addBox()
    self.timer = 0
  end
  
  -- Update every single box --
  for k, box in ipairs(self.objects) do
    if not box.removed then box:update(dt)
    else table.remove(boxes.objects, k) end
  end
end


function boxes:draw()
  -- Draw every single box --
  for _, box in ipairs(self.objects) do
    box:draw()
  end
end




--------------------------------------------------------------------------------------------
-- Other Functions --
--------------------------------------------------------------------------------------------
function checkAABBCollision(a, b)
  return a.x + a.width > b.x and a.x < b.x + b.width and
  a.y + a.height > b.y and a.y < b.y + b.height
end


function resolveCollisions(dt)
  for k, box in ipairs(boxes.objects) do
    if checkAABBCollision(player, box) then love.load() end -- reset the game 
  end
end




--------------------------------------------------------------------------------------------
-- Main callbacks --
--------------------------------------------------------------------------------------------
function love.load()
  player:init()
  boxes:init()
end



function love.update(dt)
  player:update(dt)
  boxes:update(dt)
  resolveCollisions(dt)
end


function love.draw()
  player:draw()
  boxes:draw()
  love.graphics.print("FPS: "..love.timer.getFPS(), 10, 10)
  love.graphics.print("Boxes: "..#boxes.objects, 10, 30)
  love.graphics.print("PRESS ESCAPE TO QUIT", 10, 50)
  love.graphics.print("PRESS R TO RESET", 10, 70)
end


function love.keypressed(key)
  if key == "escape" then love.event.quit()
  elseif key == "r" then love.load() end
end
